@@ -205,12 +205,14 @@ class LossContainer:
         self.number_of_skeletons: int = self.pred_skeletons.shape[0]
 
         self.joint_losses: np.ndarray = loss(self.pred_skeletons, self.gt_skeletons)
+        self.min_joint_loss_indices = np.argmin(self.joint_losses, axis=-1)
+        self.max_joint_loss_indices = np.argmax(self.joint_losses, axis=-1)
         self.skeleton_losses = np.sum(self.joint_losses, axis=-1)
         self.all_losses = np.sum(self.skeleton_losses, axis=0)
         self.min_skeleton_loss_indices = np.argmin(self.skeleton_losses, axis=0) + 1
         self.max_skeleton_loss_indices = np.argmax(self.skeleton_losses, axis=0) + 1
 
-        self.skeleton_loss_labels: List[Label] = []
+        self.skeleton_loss_labels: List[Label] = [Label(value='test')] * self.number_of_skeletons
         self.all_loss_label: Label = Label(value='test')
         self.min_loss_index_label: Label = Label(value='test')
         self.max_loss_index_label: Label = Label(value='test')
@@ -222,44 +224,45 @@ class LossContainer:
 
     def get_loss_tab(self) -> Tab:
         loss_tab: Tab = self.__create_empty_loss_tabs()
-        # TODO: fill out losses, link player value to loss indices
+        self.__set_loss_labels(0)
+        self.video_player.observe(self.__update_losses, names='value')
         return loss_tab
 
     def __create_empty_loss_tabs(self):
         skeleton_loss_tabs: List[HBox] = []
-        skeleton_losses_column: List[HBox] = []
         for i in range(self.number_of_skeletons):
-            skeleton_label: Label = Label(value='Skeleton {:d} loss:'.format(i + 1))
-            skeleton_loss: Label = Label(value='test')
-            self.skeleton_loss_labels.append(skeleton_loss)
-            skeleton_losses_column.append(HBox(children=[skeleton_label, skeleton_loss]))
-            joint_loss_labels: List[Label] = []
-            joint_losses_column: List[HBox] = []
-            for j in range(self.joint_set.number_of_joints):
-                joint_label: Label = Label(value=self.joint_set.names[j])
-                joint_loss: Label = Label(value='test')
-                joint_loss_labels.append(joint_loss)
-                joint_losses_column.append(HBox(children=[joint_label, joint_loss]))
-            joint_losses: VBox = VBox(children=joint_losses_column)
+            joint_loss_labels: List[Label] = [Label(value='test')] * self.joint_set.number_of_joints
+            joint_names: VBox = VBox(children=[Label(value=name) for name in self.joint_set.names])
+            joint_losses: VBox = VBox(children=joint_loss_labels)
             self.joint_loss_labels.append(joint_loss_labels)
-            skeleton_statistics_column: List[HBox] = []
+            skeleton_statistics_labels: List[Label] = [
+                Label(value='All losses:'),
+                Label(value='Joint with min loss:'),
+                Label(value='Joint with max loss:')]
+            skeleton_statistics_values: List[Label] = []
             all_joint_loss_label: Label = Label(value='test')
             self.all_joint_loss_labels.append(all_joint_loss_label)
-            skeleton_statistics_column.append(HBox(children=[Label(value='All losses:'), all_joint_loss_label]))
+            skeleton_statistics_values.append(all_joint_loss_label)
             min_loss_joint_label: Label = Label(value='test')
             self.min_joint_loss_name_labels.append(min_loss_joint_label)
-            skeleton_statistics_column.append(HBox(children=[Label(value='Joint with min loss:'), min_loss_joint_label]))
+            skeleton_statistics_values.append(min_loss_joint_label)
             max_loss_joint_label: Label = Label(value='test')
             self.max_joint_loss_name_labels.append(max_loss_joint_label)
-            skeleton_statistics_column.append(HBox(children=[Label(value='Joint with max loss:'), max_loss_joint_label]))
-            skeleton_statistics: VBox = VBox(children=skeleton_statistics_column)
-            skeleton_loss_tabs.append(HBox(children=[joint_losses, skeleton_statistics]))
-        skeleton_losses: VBox = VBox(children=skeleton_losses_column)
-        statistics_column: List[HBox] = [
-            HBox(children=[Label(value='All losses:'), self.all_loss_label]),
-            HBox(children=[Label(value='Min loss skeleton index:'), self.min_loss_index_label]),
-            HBox(children=[Label(value='Min loss skeleton index:'), self.max_loss_index_label])
-        ]
+            skeleton_statistics_values.append(max_loss_joint_label)
+            skeleton_labels: VBox = VBox(children=skeleton_statistics_labels)
+            skeleton_values: VBox = VBox(children=skeleton_statistics_values)
+            skeleton_loss_tabs.append(HBox(children=[joint_names, joint_losses, skeleton_labels, skeleton_values]))
+        skeleton_labels: VBox = VBox(
+            children=[Label(value='Skeleton {:d} loss:'.format(i + 1)) for i in range(self.number_of_skeletons)]
+        )
+        skeleton_losses: VBox = VBox(children=self.skeleton_loss_labels)
+        statistics_labels_column: List[Label] = [Label(value='All losses:'),
+                                                 Label(value='Min loss skeleton index:'),
+                                                 Label(value='Min loss skeleton index:')]
+        statistics_values_column: List[Label] = [self.all_loss_label,
+                                                 self.min_loss_index_label,
+                                                 self.max_loss_index_label]
+        jump_buttons: List[Button] = []
         if self.is_video:
             self.min_loss_frame_index: int = np.argmin(self.all_losses)
             min_loss_jump_button: Button = Button(description='Min loss frame')
@@ -267,14 +270,17 @@ class LossContainer:
             self.max_loss_frame_index: int = np.argmax(self.all_losses)
             max_loss_jump_button: Button = Button(description='Max loss frame')
             max_loss_jump_button.on_click(self.__jump_to_max_loss_frame)
-            statistics_column.append(HBox(children=[Label(value='Min loss frame index:'),
-                                                    Label(value=str(self.min_loss_frame_index + 1)),
-                                                    min_loss_jump_button]))
-            statistics_column.append(HBox(children=[Label(value='Max loss frame index:'),
-                                                    Label(value=str(self.max_loss_frame_index + 1)),
-                                                    max_loss_jump_button]))
-        statistics: VBox = VBox(children=statistics_column)
-        loss_tabs: List[HBox] = [HBox(children=[skeleton_losses, statistics])] + skeleton_loss_tabs
+            statistics_labels_column.append(Label(value='Min loss frame index:'))
+            statistics_labels_column.append(Label(value='Max loss frame index:'))
+            statistics_values_column.append(Label(value=str(self.min_loss_frame_index + 1)))
+            statistics_values_column.append(Label(value=str(self.max_loss_frame_index + 1)))
+            jump_buttons.append(min_loss_jump_button)
+            jump_buttons.append(max_loss_jump_button)
+        statistics_labels: VBox = VBox(children=statistics_labels_column)
+        statistics_values: VBox = VBox(children=statistics_values_column)
+        loss_tabs: List[HBox] = [HBox(children=[
+            skeleton_labels, skeleton_losses, statistics_labels, statistics_values
+        ] + ([VBox(children=jump_buttons)]) if len(jump_buttons) > 0 else [] )] + skeleton_loss_tabs
         loss_tab = Tab(children=loss_tabs)
         loss_tab.set_title(0, "All losses")
         for i in range(len(loss_tabs)):
@@ -289,10 +295,18 @@ class LossContainer:
 
     def __set_loss_labels(self, index: int) -> None:
         for i in range(self.number_of_skeletons):
-            pass
+            self.skeleton_loss_labels[i].value = '{:.3f}'.format(self.skeleton_losses[i][index])
+            for j in range(self.joint_set.number_of_joints):
+                self.joint_loss_labels[i][j].value = '{:.3f}'.format(self.joint_losses[i][index][j])
+            self.all_joint_loss_labels[i].value = '{:.3f}'.format(self.skeleton_losses[i][index])
+            self.min_joint_loss_name_labels[i].value = self.joint_set.names[self.min_joint_loss_indices[i][index]]
+            self.max_joint_loss_name_labels[i].value = self.joint_set.names[self.max_joint_loss_indices[i][index]]
+        self.all_loss_label.value = '{:.3f}'.format(self.all_losses[index])
+        self.min_loss_index_label.value = str(self.min_skeleton_loss_indices[index])
+        self.max_loss_index_label.value = str(self.max_skeleton_loss_indices[index])
 
     def __update_losses(self, current_frame):
-        a = current_frame.new
+        self.__set_loss_labels(current_frame.new)
 
 
 class SkeletonVisualizer:
